﻿//Ciekawe dodatki

module Demo6

open System

// Units of measure
// ---------------------------------------------------------------

// Mass, grams
[<Measure>] type g
// Mass, kilograms
[<Measure>] type kg
// Weight, pounds
[<Measure>] type lb 

// Distance, meters 
[<Measure>] type m
// Distance, cm
[<Measure>] type cm

// Distance, inches
[<Measure>] type inch
// Distance, feet
[<Measure>] type ft

// Time, seconds
[<Measure>] type s

// Force, Newtons
[<Measure>] type N = kg m / s 

// Pressure, bar
[<Measure>] type bar 
// Pressure, Pascals
[<Measure>] type Pa = N / m^2 

// Volume, milliliters
[<Measure>] type ml 
// Volume, liters
[<Measure>] type L

// Define conversion constants.
let gramsPerKilogram : float<g kg^-1> = 1000.0<g/kg>
let cmPerMeter : float<cm/m> = 100.0<cm/m>
let cmPerInch : float<cm/inch> = 2.54<cm/inch>

let mlPerCubicCentimeter : float<ml/cm^3> = 1.0<ml/cm^3>
let mlPerLiter : float<ml/L> = 1000.0<ml/L>

// Define conversion functions.
let convertGramsToKilograms (x : float<g>) = x / gramsPerKilogram
let convertCentimetersToInches (x : float<cm>) = x / cmPerInch

[<Measure>] type C // temperature, Celsius/Centigrade
[<Measure>] type F // temperature, Fahrenheit

let convert_CtoF ( temp : float<C> ) = 9.0<F> / 5.0<C> * temp + 32.0<F>
let convert_FtoC ( temp: float<F> ) = 5.0<C> / 9.0<F> * ( temp - 32.0<F>)

// Define a vector together with a measure type parameter.
// Note the attribute applied to the type parameter.
type vector3D<[<Measure>] 'u> = { x : float<'u>; y : float<'u>; z : float<'u>}

// Create instances that have two different measures.
// Create a position vector.
let xvec : vector3D<m> = { x = 0.0<m>; y = 0.0<m>; z = 0.0<m> }
// Create a velocity vector.
let v1vec : vector3D<m/s> = { x = 1.0<m/s>; y = -1.0<m/s>; z = 0.0<m/s> }

// Async monads - Parallel primes
// ---------------------------------------------------------------

(* very naive prime number detector *)
let is_prime (n:int) =
 let bound = int (System.Math.Sqrt(float n)) in
  {2 .. bound} |> Seq.exists (fun x -> n % x = 0) |> not
 
(* we are using async workflows *)
let primeAsync n =
    async { return (n, is_prime n) }
 
(* return primes between m and n using multiple threads *)  
let primes m n =
    {m .. n}
        |> Seq.map primeAsync
        |> Async.Parallel
        |> Async.RunSynchronously
        |> Array.filter snd
        |> Array.map fst
 
(* run a test *)
primes 1000000 1002000
    |> Array.iter (printfn "%d")
    
//lookup: http://msdn.microsoft.com/en-us/library/dd233182(VS.100).aspx

// Code quotations
// ---------------------------------------------------------------

<@ 2 + 2 @> |> ignore